#!/usr/bin/perl

# simple tool to check the iXhash hashes against the currently known DNS zones.
# should work but is a crude hack only
# Usage: ./hashcheck.pl < test.msg
#
#

my @msg = <>;

use Digest::MD5 qw(md5 md5_hex md5_base64);
use POSIX qw(locale_h);
use Net::DNS;
use Net::DNS::Resolver;

my $rr;
my $resolver = Net::DNS::Resolver->new;
# zones to query against. If you intend to query other ones plug them in here
my @zonen = ( "generic.ixhash.net",
	"ctyme.ixhash.net",
	"ix.dnsbl.manitu.net",
	"hosteurope.ixhash.net");
	
foreach (@msg) {
	$msg .= join "", $_;
}

# split $msg at empty line
my ($header,$body) = split(/\n\n/,$msg,2);


# Now compute the hashes, along the lines used in the iXhash plugin
if (($body =~ /(?>\s.+?){20}/g) || ($body =~ /\n.*\n/)) {
             
	$body_copy = $body;
	# Step One
	$body_copy =~ s/([[:space:]]{100})(?:\1+)/$1/g;
	# Step Two
	$body_copy =~ s/([[:space:]])(?:\1+)/$1/g;
	# remove graph class chars and some specials
	$body_copy =~ s/[[:graph:]]+//go;
	# Create actual digest
	$digest = md5_hex($body_copy);
	foreach $dnszone (@zonen){
		$answer = $resolver->search($digest.'.'.$dnszone, "A", "IN");
		if ($answer) {
			foreach $rr ($answer->answer) {
				next unless $rr->type eq "A";
				$hits = $rr->address;
				$hits =~ /^127\.d{1,3}\.d{1,3}\.d{1,3}/;
				print "Hit at ". $digest.'.'.$dnszone.": Returned $hits\n"
			}
		}
		print "Computed IXHASH #1 $digest; No hit found at $dnszone \n" unless $answer;
	}
}
else
{
	print "IXHASH #1: Hash value not computed, requirements not met\n";
}
# Now advance to hash # 2
# Creation of hash # 2 if mail contains at least 3 of the following characters:
# '[<>()|@*'!?,]' or the combination of ':/'
# (To match something like "Already seen?  http:/host.domain.tld/")
if ($body =~ /((([<>\(\)\|@\*'!?,])|(:\/)).*?){3,}/m ) {
	$body_copy = $body;
	# remove redundant stuff
	$body_copy =~ s/[[:cntrl:][:alnum:]%&#;=]+//g;
	# replace '_' with '.'
	$body_copy =~ tr/_/./;
	# replace duplicate chars. This too suffers from a bug in perl
	# so we do it in two steps
	# Step One
	$body_copy =~ s/([[:print:]]{100})(?:\1+)/$1/g;
	# Step Two
	$body_copy =~ s/([[:print:]])(?:\1+)/$1/g;
	# Computing hash...
	$digest = md5_hex($body_copy);
	$answer = $resolver->search($digest.'.'.$dnszone, "A", "IN");
	foreach $dnszone (@zonen){
		$answer = $resolver->search($digest.'.'.$dnszone, "A", "IN");
		if ($answer) {
			foreach $rr ($answer->answer) {
				next unless $rr->type eq "A";
				$hits = $rr->address;
				$hits =~ /^127\.d{1,3}\.d{1,3}\.d{1,3}/;
				print "Hit at ". $digest.'.'.$dnszone.": Returned $hits\n"
			}
		}
		print "Computed IXHASH #2 $digest; No hit found at $dnszone \n" unless $answer;
	}
}
else
{
	print "IXHASH #2: Hash value not computed, requirements not met\n";
}

#Finally hash #3
if ($body =~ /[\S]{8}/) {
	$body_copy = $body;
	$body_copy =~ s/[[:cntrl:][:space:]=]+//g;
	# replace duplicate chars. This too suffers from a bug in perl
	# so we do it in two steps
	# Step One
	$body_copy =~ s/([[:print:]]{100})(?:\1+)/$1/g;
	# Step Two
	$body_copy =~ s/([[:graph:]])(?:\1+)/$1/g;
	# Computing actual hash
	$digest = md5_hex($body_copy);
	foreach $dnszone (@zonen){
		$answer = $resolver->search($digest.'.'.$dnszone, "A", "IN");
		if ($answer) {
			foreach $rr ($answer->answer) {
				next unless $rr->type eq "A";
				$hits = $rr->address;

				$hits =~ /^127\.d{1,3}\.d{1,3}\.d{1,3}/;
				print "Hit at ". $digest.'.'.$dnszone.": Returned $rr->address\n"
			}
		}
		print "Computed IXHASH #3 $digest; No hit found at $dnszone \n" unless $answer;
	}
}
else
{
	print "IXHASH #3: Hash value not computed, requirements not met\n";
}
